# "Trouble Globe" USB status light

This is an archived AVR-C project that enables control of WS2811 LEDs over USB
using the V-USB library. It was originally written for an ATMega328P.

RGB values are sent using USB control writes and the commands/args defined in
`commands.h`. See the Python demo below for a practical use example.

The project requires `avr-gcc` to build and `avrdude` to program a chip. This
is set up in the Makefile:

```bash
# Build the project
make

# Build and program a device
make flash
```

The build expects the AVR to be fused to run from a 16MHz external oscillator,
but V-USB supports a 12MHz crystal as well.

## Python demo

The file `status_globe.py` is an example host application using Python to
control the status light.

This requires the package [libusb1](https://pypi.python.org/pypi/libusb1) to work.

### Allowing control without root privileges

Under Linux the device may need a udev rule added so it can be controlled by a
non-root user:

```bash
# Create rule so the device can be controlled by any user
echo 'SUBSYSTEM=="usb", ATTR{idVendor}=="f055", ATTR{idProduct}=="05df", MODE="0666"' \
  | tee /etc/udev/rules.d/99-status-light.rules

# Reload new udev rule
# Plug the device in after this is run, or reconnect if already connected
udevadm control --reload-rules && udevadm trigger
```

## Licence

```
Copyright 2016-2019 Stephen Holdaway

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
```
