#include "StatusLight.h"
#include "../firmware/commands.h"
#include <iostream>
#include <cstring>

#define USB_VENDOR 0xF055 // "FOSS" - generic defacto vendor
#define USB_PRODUCT 0x05DF // Generic HID, not keyboard, mouse etc
#define DEVICE_SERIAL "STCNZ001"

#define USB_TIMEOUT_MS 5000
#define USB_RQ 0x04

StatusLight::StatusLight()
{
	libusb_init(&usbContext);
	libusb_set_debug(usbContext, 2);

	deviceHandle = _getDevice();

	if (!deviceHandle) {
		std::cerr << "Didn't find any device matching the status globe :(" << std::endl;
	} else {
		std::cout << "GOT THE BASTARD" << std::endl;
		libusb_claim_interface(deviceHandle, 0);
	}
}

StatusLight::~StatusLight()
{
	if (deviceHandle != NULL) {
		libusb_close(deviceHandle);
	}

	libusb_exit(usbContext);
}

bool StatusLight::isConnected()
{
	return deviceHandle != NULL;
}

void StatusLight::setColour(uint8_t r, uint8_t g, uint8_t b)
{
	uint8_t buffer[] = {
		LIGHT_CMD_SET_ALL,
		r,
		g,
		b
	};

	_sendControlData(buffer, (uint8_t) sizeof(buffer));
}

void StatusLight::setLights(uint8_t r1, uint8_t g1, uint8_t b1, uint8_t r2, uint8_t g2, uint8_t b2, uint8_t r3, uint8_t g3, uint8_t b3)
{
	uint8_t buffer[] = {
		LIGHT_CMD_SET_EACH,
		r1, g1, b1,
		r2, g2, b2,
		r3, g3, b3
	};

	_sendControlData(buffer, (uint8_t) sizeof(buffer));
}

int StatusLight::_sendControlData(uint8_t buf[], uint8_t length)
{
	return libusb_control_transfer(
		deviceHandle,
		LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_OUT,
		LIBUSB_REQUEST_SET_CONFIGURATION,
		0,
		0,
		buf,
		length,
		0
	);
}

libusb_device_handle* StatusLight::_getDevice() {
	libusb_device **devices;
	int status;
	size_t numDevices = libusb_get_device_list(usbContext, &devices);

	// Look through all USB devices for ours
	for (int i = numDevices - 1; i >= 0; i--) {
		struct libusb_device_descriptor descriptor;
		libusb_get_device_descriptor(devices[i], &descriptor);

		// Match the vendor and product the status light uses
		if (descriptor.idVendor == USB_VENDOR && descriptor.idProduct == USB_PRODUCT) {
			libusb_device_handle *handle;
			status = libusb_open(devices[i], &handle);

			if (status < 0) {
				std::cerr << "Failed to open USB device. Do you have the right permissions?\n";
				std::cerr << "If you're on Linux, you'll need the udev rule:\n  ";
				std::cerr << "SUBSYSTEMS==\"usb\", ATTRS{idVendor}==\"F055\", ATTRS{idProduct}==\"05DF\", SYMLINK+=\"sub20\", GROUP=\"usb\", MODE=\"660\"\n";
				continue;
			}

			unsigned char serial[256];
			libusb_get_string_descriptor_ascii(handle, descriptor.iSerialNumber, serial, 256);

			if (strcmp(DEVICE_SERIAL, reinterpret_cast<const char*>(serial)) == 0) {
				return handle;
			} else {
				libusb_close(handle);
			}
		}
	}

	return NULL;
}