/* USB control commands for the status light */

/**
 * Set all lights to the same colour
 *
 * @param { uint8_t r, g, b }
 */
#define LIGHT_CMD_SET_ALL 0xC0

/**
 * Set the colour of each LED group in sequence
 *
 * @param { uint8_t r, g, b, r, g, b, ... }
 */
#define LIGHT_CMD_SET_EACH 0xC1

/**
 * Load a sequence of frames into the buffer
 *
 * NOTE: Not implemented
 */
#define LIGHT_CMD_PUSH_SEQUENCE 0xC2

/**
 * Enable or disable automatic slow fade to target colour
 * @param { uint8_t feature, bool enabled }
 * @see LIGHT_FEATURE_*
 */
#define LIGHT_CMD_SET_FEATURE 0xCE

/**
 * Set the automatic decoration mode of the LEDs
 *
 * @param { uint8_t mode }
 * @see LIGHT_MODE_*
 */
#define LIGHT_CMD_SET_MODE 0xCF


/**
 * Light modes
 * @see LIGHT_CMD_SET_MODE
 */

// Stay on the selected colours, unchanging
#define LIGHT_MODE_STATIC 0x0

// Brightness pulses in the middle of the light that propagate out
#define LIGHT_MODE_BREATHE 0x1

// Loop the sequence loaded with the SET_SEQUENCE command
// Frames are played back at 60Hz
#define LIGHT_MODE_LOOP_SEQUENCE 0x2


/**
 * Light features
 * @see LIGHT_CMD_SET_FEATURE
 */

// Fade slowly to a colour instead of instantly changing
#define LIGHT_FEATURE_TRANSITION 0x1

// Correct the brightness of each colour to roughly match an SRGB display
#define LIGHT_FEATURE_CALIBRATION 0x2

// Map the input brightness to apparent brightness
// This makes linear ramps of RGB value appear smooth to a human observer
#define LIGHT_FEATURE_CIE_BRIGHTNESS 0x4
