#include "ws2812_helpers.h"

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

void cAdd(cRGB *colour, int8_t value)
{
	if (value > 0) {
		colour->b += MIN(255 - colour->b, value);
		colour->r += MIN(255 - colour->r, value);
		colour->g += MIN(255 - colour->g, value);
	} else {
		colour->b -= MIN(colour->b, -value);
		colour->r -= MIN(colour->r, -value);
		colour->g -= MIN(colour->g, -value);
	}
}

void cMult(cRGB *colour, float value)
{
	colour->b *= MIN(255.0/colour->b, value);
	colour->r *= MIN(255.0/colour->r, value);
	colour->g *= MIN(255.0/colour->g, value);
}

void applyCieBrightness(cRGB *colour)
{
	// CIE1931 luminance correction table
	// Generated from code here: http://jared.geek.nz/2013/feb/linear-led-pwm
	static const __flash unsigned char CIE_CORRECTION[256] = {
		0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
		1, 1, 1, 1, 2, 2, 2, 2, 2, 2,
		2, 2, 2, 3, 3, 3, 3, 3, 3, 3,
		3, 4, 4, 4, 4, 4, 4, 5, 5, 5,
		5, 5, 6, 6, 6, 6, 6, 7, 7, 7,
		7, 8, 8, 8, 8, 9, 9, 9, 10, 10,
		10, 10, 11, 11, 11, 12, 12, 12, 13, 13,
		13, 14, 14, 15, 15, 15, 16, 16, 17, 17,
		17, 18, 18, 19, 19, 20, 20, 21, 21, 22,
		22, 23, 23, 24, 24, 25, 25, 26, 26, 27,
		28, 28, 29, 29, 30, 31, 31, 32, 32, 33,
		34, 34, 35, 36, 37, 37, 38, 39, 39, 40,
		41, 42, 43, 43, 44, 45, 46, 47, 47, 48,
		49, 50, 51, 52, 53, 54, 54, 55, 56, 57,
		58, 59, 60, 61, 62, 63, 64, 65, 66, 67,
		68, 70, 71, 72, 73, 74, 75, 76, 77, 79,
		80, 81, 82, 83, 85, 86, 87, 88, 90, 91,
		92, 94, 95, 96, 98, 99, 100, 102, 103, 105,
		106, 108, 109, 110, 112, 113, 115, 116, 118, 120,
		121, 123, 124, 126, 128, 129, 131, 132, 134, 136,
		138, 139, 141, 143, 145, 146, 148, 150, 152, 154,
		155, 157, 159, 161, 163, 165, 167, 169, 171, 173,
		175, 177, 179, 181, 183, 185, 187, 189, 191, 193,
		196, 198, 200, 202, 204, 207, 209, 211, 214, 216,
		218, 220, 223, 225, 228, 230, 232, 235, 237, 240,
		242, 245, 247, 250, 252, 255,
	};

	colour->b = CIE_CORRECTION[colour->b];
	colour->r = CIE_CORRECTION[colour->r];
	colour->g = CIE_CORRECTION[colour->g];
}

void applyColourCalibration(cRGB *colour, float redFactor, float greenFactor, float blueFactor)
{
	colour->b *= blueFactor;
	colour->r *= redFactor;
	colour->g *= greenFactor;
}

/**
 * Fade {colour} to {target} using only 8 bit values
 *
 * This might be faster using a 16 bit value to calculate the delta, since that
 * would require less branching.
 */
int8_t cFadeTo(cRGB *colour, cRGB *target, float factor)
{
    int8_t changed = 0;
    uint8_t diff = 0;

    if (colour->b != target->b) {
        if (colour->b > target->b) {
            diff = (colour->b - target->b);
            colour->b -= diff * factor ?: 1;
        } else {
            diff = target->b - colour->b;
            colour->b += diff * factor ?: 1;
        }

        changed++;
    }

    if (colour->r != target->r) {
        if (colour->r > target->r) {
            diff = (colour->r - target->r);
            colour->r -= diff * factor ?: 1;
        } else {
            diff = target->r - colour->r;
            colour->r += diff * factor ?: 1;
        }

        changed++;
    }

    if (colour->g != target->g) {
        if (colour->g > target->g) {
            diff = (colour->g - target->g);
            colour->g -= diff * factor ?: 1;
        } else {
            diff = target->g - colour->g;
            colour->g += diff * factor ?: 1;
        }

        changed++;
    }

    return changed;
}
