
/* Name: ws2812_helpers.c
 * Author: Stephen Holdaway <stephen@stecman.co.nz>
 * Date: 20 November, 2016
 * License: MIT
 */
#pragma once

#include <avr/sfr_defs.h>

// The colour wiring in this project differs from the WS2811S chips being
// used in order to simplify routing for a single-sided PCB layout
typedef struct cRGB {
	uint8_t b;
	uint8_t r;
	uint8_t g;
} cRGB;

/**
 * Create and configure a colour structure
 */
inline cRGB RGB(uint8_t r, uint8_t g, uint8_t b)
{
	return (cRGB) { b, r, g };
}

/**
 * Add a value to each component of a colour
 */
void cAdd(cRGB *colour, int8_t value);

/**
 * Multiply each component of a color by a value
 * Passing a negative factor is undefined  isxpected
 */
void cMult(cRGB *colour, float factor);

/**
 * Transform a colour value to CIE colour space (apparent linear brightness)
 */
void applyCieBrightness(cRGB *colour);

/**
 * Multiply each colour component by a different value to correct apparent colour
 */
void applyColourCalibration(cRGB *colour, float redFactor, float greenFactor, float blueFactor);

/**
 * Fade one colour to another
 *
 * The change per frame is (difference * {factor})
 *
 * @return >=1 if colour was changed, 0 when the fade is complete
 */
int8_t cFadeTo(cRGB *colour, cRGB *target, float factor);
